# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rcpp_fft_axis <- function(arr, dims, axis, inverse = FALSE) {
    .Call(`_fastlpr_rcpp_fft_axis`, arr, dims, axis, inverse)
}

rcpp_fft_nd <- function(arr, dims, inverse = FALSE) {
    .Call(`_fastlpr_rcpp_fft_nd`, arr, dims, inverse)
}

rcpp_fft_nd_real <- function(arr, dims, inverse = FALSE) {
    .Call(`_fastlpr_rcpp_fft_nd_real`, arr, dims, inverse)
}

rcpp_interp_nd <- function(grid_vals, arr, points) {
    .Call(`_fastlpr_rcpp_interp_nd`, grid_vals, arr, points)
}

rcpp_interp_nd_complex <- function(grid_vals, arr, points) {
    .Call(`_fastlpr_rcpp_interp_nd_complex`, grid_vals, arr, points)
}

rcpp_aperm <- function(arr, dims, perm) {
    .Call(`_fastlpr_rcpp_aperm`, arr, dims, perm)
}

rcpp_aperm_complex <- function(arr, dims, perm) {
    .Call(`_fastlpr_rcpp_aperm_complex`, arr, dims, perm)
}

rcpp_get_num_threads <- function() {
    .Call(`_fastlpr_rcpp_get_num_threads`)
}

rcpp_set_num_threads <- function(n) {
    invisible(.Call(`_fastlpr_rcpp_set_num_threads`, n))
}

rcpp_interp_batch_nd <- function(grid_vectors, values, dims, query_points) {
    .Call(`_fastlpr_rcpp_interp_batch_nd`, grid_vectors, values, dims, query_points)
}

rcpp_fft3d_batch <- function(arr, n1, n2, n3, dh, inverse = FALSE) {
    .Call(`_fastlpr_rcpp_fft3d_batch`, arr, n1, n2, n3, dh, inverse)
}

rcpp_fft2d_batch <- function(arr, n1, n2, n_slices, inverse = FALSE) {
    .Call(`_fastlpr_rcpp_fft2d_batch`, arr, n1, n2, n_slices, inverse)
}

rcpp_broadcast_multiply <- function(kdf, y_ft, L, dh, dy) {
    .Call(`_fastlpr_rcpp_broadcast_multiply`, kdf, y_ft, L, dh, dy)
}

rcpp_broadcast_multiply_nd <- function(kdf, y_ft, spatial_dims, dh, dy) {
    .Call(`_fastlpr_rcpp_broadcast_multiply_nd`, kdf, y_ft, spatial_dims, dh, dy)
}

rcpp_extract_subarray <- function(arr, dims, starts, ends) {
    .Call(`_fastlpr_rcpp_extract_subarray`, arr, dims, starts, ends)
}

rcpp_conv_nd_full <- function(kdf, y_ft, L_vec, dh, dy, qout, y_isreal = TRUE) {
    .Call(`_fastlpr_rcpp_conv_nd_full`, kdf, y_ft, L_vec, dh, dy, qout, y_isreal)
}

rcpp_gaussian_kernel_grid <- function(xgrid, h) {
    .Call(`_fastlpr_rcpp_gaussian_kernel_grid`, xgrid, h)
}

rcpp_nufft_type1 <- function(knot, y, L_vec, accuracy) {
    .Call(`_fastlpr_rcpp_nufft_type1`, knot, y, L_vec, accuracy)
}

rcpp_nufft_type1_complex <- function(knot, y, L_vec, accuracy) {
    .Call(`_fastlpr_rcpp_nufft_type1_complex`, knot, y, L_vec, accuracy)
}

rcpp_scale_knots <- function(knot, N, Fs) {
    .Call(`_fastlpr_rcpp_scale_knots`, knot, N, Fs)
}

rcpp_nufft_binning <- function(knot, y, Mr_vec) {
    .Call(`_fastlpr_rcpp_nufft_binning`, knot, y, Mr_vec)
}

rcpp_nufft_binning_complex <- function(knot, y, Mr_vec) {
    .Call(`_fastlpr_rcpp_nufft_binning_complex`, knot, y, Mr_vec)
}

rcpp_info <- function() {
    .Call(`_fastlpr_rcpp_info`)
}

