% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcpp_wrappers.R
\name{get_rcpp_info}
\alias{get_rcpp_info}
\title{Get Rcpp Information}
\usage{
get_rcpp_info()
}
\value{
A list with Rcpp version and capabilities if available, NULL otherwise.
}
\description{
Returns information about the Rcpp backend including version
and available acceleration features.
}
\examples{
get_rcpp_info()
}
